#include "SDLV110.h"

#include <cstdio>
#include <cmath>

BASICENGINE::BASICENGINE(VESSEL* pVessel)
{
	pParent = pVessel;
	fIgnited = false;
	fPitch = 0;
	fYaw = 0;
	fPitch0 = 0;
	fYaw0 = 0;
	fPitchCmd = 0;
	fYawCmd = 0;
	fRealized = false;
	fIsIgniting = false;
}

BASICENGINE::~BASICENGINE()
{
}

void BASICENGINE::CalcEngineParameters(double fSimT, double fDeltaT)
{
}

void BASICENGINE::Create(VECTOR3& pos, VECTOR3& Dir, PROPELLANT_HANDLE phSource)
{
	pos0 = pos;
	dir0 = Dir;
}

void BASICENGINE::Destroy()
{
	Unrealize();
}

void BASICENGINE::DoEngineController(double fSimT, double fDeltaT)
{
}

void BASICENGINE::DoTVC(double fSimT, double fDeltaT)
{
	if(IsRealized())
	{
		if(fPitch != fPitchCmd || fYaw != fYawCmd)
		{
			double fTotalDefl = sqrt(pow(fPitchCmd, 2) + pow(fYawCmd, 2));
			double dX, dY;
			//Fix for Movement Limits;
			if(fTotalDefl > MAX_TVC) 
			{	
				dX = MAX_TVC * (fYawCmd)/fTotalDefl - fYaw;
				dY = MAX_TVC * (fPitchCmd)/fTotalDefl - fPitch;
			} else 
			{
				dX = fYawCmd - fYaw;
				dY = fPitchCmd - fPitch;
			}
			if(pow(dX, 2) + pow(dY, 2) > pow(MAX_TVC_RATE, 2) * fDeltaT)
			{
				double L = sqrt(pow(dX, 2) + pow(dY, 2));
				dX = MAX_TVC_RATE * fDeltaT * dX/L;
				dY = MAX_TVC_RATE * fDeltaT * dY/L;
			} 
			fYaw += dX;
			fPitch += dY;
		}
	}
}


void BASICENGINE::FireIgniter()
{
	if(iIgniterCount > 0)
	{
		iIgniterCount--;
		fIsIgniting = true;
		fIgnited = CheckIgnitionConditions();
	}
}

VECTOR3 BASICENGINE::GetPosition() const
{
	return pos0;
}

bool BASICENGINE::CheckIgnitionConditions() const
{
	return fIgnited;
}

void BASICENGINE::clbkPostStep(double fSimT, double fDeltaT)
{
	if(IsRealized())
	{
		DoEngineController(fSimT, fDeltaT);
		DoTVC(fSimT, fDeltaT);
		fIgnited = CheckIgnitionConditions();
		CalcEngineParameters(fSimT,fDeltaT);
		fIsIgniting = false;
	}
}



double BASICENGINE::GetThrustCommand() const
{
	return fThrustCmd;
}

VECTOR3 BASICENGINE::GetThrustVector() const
{
	return _V(sin(fYaw + fYaw0), sin(fPitch + fPitch0), cos(fYaw + fYaw0) * cos(fPitch + fPitch0));
}

double BASICENGINE::GetXDeflection() const
{
	return fYaw;
}

double BASICENGINE::GetYDeflection() const
{
	return fPitch;
}

void BASICENGINE::Ignite()
{
	if(IsRealized())
	{
		FireIgniter();
	}
}

bool BASICENGINE::IgniterFired() const
{
	if(IsRealized())
		return fIsIgniting;
	else
		return false;
}

bool BASICENGINE::IsIgnited() const
{
	return fIgnited;
}

bool BASICENGINE::IsRealized() const
{
	return fRealized;
}

void BASICENGINE::Realize()
{
	fRealized = true;
	iIgniterCount = MAX_IGNITIONS;
}

void BASICENGINE::SetMaximumNumberOfIgniters(int iMax)
{
	MAX_IGNITIONS = iMax;
}

void BASICENGINE::SetNeutralThrustVector(VECTOR3& vtn)
{
	this->dir0 = vtn;
	fPitch0 = atan2(dir0.y,dir0.z);
	fYaw0 = atan2(dir0.x,dir0.z);
}


void BASICENGINE::SetThrustDir(VECTOR3 newDir)
{
		fPitchCmd = atan2(newDir.y, newDir.z);
		fYawCmd = atan2(newDir.x, newDir.z);	
}

void BASICENGINE::SetThrustLevel(double fThrustLevel)
{
	fThrustCmd = fThrustLevel;
}


void BASICENGINE::SetTVCLimits(double fMaxTVC, double fMaxTVCRate)
{
	MAX_TVC = fMaxTVC;
	MAX_TVC_RATE = fMaxTVCRate;
}

void BASICENGINE::Unrealize()
{
	fRealized = false;
}

