#include "SDLV110.h"
#include <cmath>
#include <cstdio>

RSRB5::RSRB5(VESSEL* pVessel)
: BASICENGINE(pVessel)
{
	thMe = NULL;
	fNextLogTime = 0.0;
	fBurnTime = 0.0;
}

RSRB5::~RSRB5()
{
}

void RSRB5::Create(VECTOR3& pos, VECTOR3& dir0, PROPELLANT_HANDLE phSource)
{
	if(!IsRealized())
	{
		SURFHANDLE shSRBTex = oapiRegisterExhaustTexture("exhaust_atsrb");
	
		PARTICLESTREAMSPEC srb_contrail = {
			0, 12.0, 3, 150.0, 0.4, 8.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
			PARTICLESTREAMSPEC::ATM_PLOG, 0.001, 0.1
		};

		PARTICLESTREAMSPEC srb_contrail2 = {
			0, 12.0, 3, 150.0, 0.4, 8.0, 6, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.001
		};
		PARTICLESTREAMSPEC srb_exhaust = {
			0, 3.0, 40, 250.0, 0.04, 0.7, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
			PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
		};

		srb_exhaust.tex = oapiRegisterParticleTexture ("Contrail2");
		srb_contrail.tex = oapiRegisterParticleTexture ("Contrail4");
		srb_contrail2.tex = oapiRegisterParticleTexture ("Contrail4");
		BASICENGINE::Create(pos, dir0, phSource);

		thMe = pParent->CreateThruster( pos, dir0, RSRB5_THRUST0, 
				phSource, RSRB5_ISP0, RSRB5_ISPSL);

		phSRB = phSource;

		VECTOR3 vX = pos + _V(0.0, 0.0, -6);
		pParent->AddExhaust(thMe, 40.0, 3.8, 0, shSRBTex);
		pParent->AddExhaustStream(thMe, vX, &srb_exhaust);
		vX.z -= 55.0;
		pParent->AddExhaustStream(thMe, vX, &srb_contrail);
		//pParent->AddExhaustStream(thMe, vX, &srb_contrail2);

		SetMaximumNumberOfIgniters(4);
		SetNeutralThrustVector(dir0);
		SetTVCLimits(8.0 * RAD, 8.0 * RAD);
		Realize();
	}
}

void RSRB5::Destroy()
{
	if(IsRealized())
	{
		pParent->DelThruster(thMe);
		thMe = NULL;
		phSRB = NULL;
		Unrealize();
	}
}

void RSRB5::CalcEngineParameters(double fSimT, double fDeltaT)
{
	const double MDOTMAX = 6485.6;
	if(IsRealized())
	{
		pParent->SetThrusterDir(thMe, GetThrustVector());
		if(IsIgnited())
		{
			double gamma = 0.0;
			fBurnTime += fDeltaT;
			if(fBurnTime > 400 )
			{
				gamma = 0.0;
				fIgnited = false;
			}
			if(fBurnTime > 128)
			{
				gamma = 0.015 - 0.015 * (fBurnTime - 128)/272.0;
			}
			else if (fBurnTime > 110.0)
			{
				gamma = 0.5 - 0.485 * (fBurnTime - 110)/18.0;
			} else if(fBurnTime > 80.0)
			{
				gamma = 0.75 - 0.25 * (fBurnTime - 80.0)/30.0;
			} else if(fBurnTime > 50.0)
			{
				gamma = 0.66 + 0.09 * (fBurnTime - 50.0)/30.0;
			} else if(fBurnTime > 20.0)
			{
				gamma = 1.0 - 0.34 * (fBurnTime - 20.0) / 30.0;
			} else
			{
				gamma = 0.9 + 0.1 * fBurnTime/20.0;
			}
			

			if(fSimT > fNextLogTime)
			{
				fNextLogTime = fSimT + 1.0;
				FILE* pLog = fopen("RSRB5.log", "at");
				if(pLog)
				{
					fprintf(pLog, "[%f]\tgamma = %f\ttb = %f\t\n", 
						fSimT, gamma, fBurnTime);
					fclose(pLog);
				}
			}

			pParent->SetThrusterLevel(thMe, gamma);
	
		} else {
			pParent->SetThrusterLevel(thMe, 0.0);
		}
	}
}

bool RSRB5::CheckIgnitionConditions() const
{
	return (IsIgnited() || IgniterFired());
}

void RSRB5::Ignite()
{
	if(IsRealized())
	{
		FireIgniter();
		FireIgniter();
		FireIgniter();
	}
}

void RSRB5::SetRemainingBurnTime(double fBT)
{
	fBurnTime = fBT;
	if(fBT> 0.0)
	{
		fIgnited = true;
	}
}
