#ifndef ___RSRB5_H_INCLUDED___
#define ___RSRB5_H_INCLUDED___

#include "OrbiterAPI.h"
#include "VesselAPI.h"
#include "SDLV_Engines.h"

static const char* MESHNAME_SDLV_RSRB5_LEFT = "sdlv/sdlv_rsrb5l";
static const char* MESHNAME_SDLV_RSRB5_RIGHT = "sdlv/sdlv_rsrb5r";

const double SDLV_RSRB5_EMPTYMASS = 87.0E3;
const double SDLV_RSRB5_PROPELLANTMASS = 621.0E3;


const double BSM_THRUST0 = 10.0E4;

/**
 * Tilt one BSM of the first lower group down slightly to create a upward pitch
 * @todo Missing meshes
 * - Pilot chute
 * - Drogue chute
 * - main chutes 
 * - Water line
 */
class SDLV_RSRB5: public VESSEL2
{
	bool fIsLeft;
	/**
	 * 0 = Post separation
	 * 1 = After BSM burns
	 * 2 = Baroswitches armed / apogee
	 * 3 = Nose cone separated, pilot deployed
	 * 4 = Drogue deployed
	 * 5 = Frustrum separated, mains deploying
	 * 6 = mains deployed
	 * 7 = After noozle extension had been separated
	 * 8 = watered, floating
	 */
	int iState;
	RSRB5* pSRB;
	PROPELLANT_HANDLE phSRB;
	PROPELLANT_HANDLE phBSM1;
	PROPELLANT_HANDLE phBSM2;
	double fTimer;
	double fDrogueState;
	double fMainState;
	THRUSTER_HANDLE thBSM[8];
protected:
	void CreateBSMThrusters();
	void IgniteGroup1();
	void IgniteGroup2();
	void SeparateNoseCone();
	void DeployDrogue();
	void SeparateFrustum();
	void DeployMain();
	void SeparateNoozleExt();
	void DeployFloatationAid();
	void SetupPostSep();
	void SetupDrogue();
	void SetupMainChutes();
	void SetupNoNoozleEx();
	void SetupLanded();
public:
	SDLV_RSRB5(OBJHANDLE hVessel, int iFlightModel);
	virtual ~SDLV_RSRB5();
	virtual void clbkLoadStateEx(FILEHANDLE scn, void *vs);
	virtual void clbkSaveState(FILEHANDLE scn);
	virtual void clbkSetStateEx(const void* status);
	virtual void clbkSetClassCaps(FILEHANDLE cfg);
	virtual void clbkPostStep(double fSimT, double fDeltaT, double fMJD);
	void SetRemainingBurnTime(double fBT);
	void SetRemainingPropellant(double fPropMass);
};

#endif