#include "SDLV110.h"
#include <cstdio>

SDLV110::SDLV110(OBJHANDLE hVessel, int iFlightModel)
	: VESSEL2(hVessel, iFlightModel),
	PitchControl(0.08 * RAD, 5.0, 4.0),
	YawControl(0.08 * RAD, 5.0, 4.0),
	BankControl(0.2 * RAD, 25.0, 2.00),
	PitchRateControl(1.1, 15.0, 0.05),
	BankRateControl(2.0, 10.0, 0.05),
	YawRateControl(-1.1, 10.0, 0.005)
{
	fairing_type = FAIRING_NONE;
	phFirst = NULL;
	phSecond = NULL;
	phSecondRCS = NULL;
	
	phRSRB5[0] = NULL;
	phRSRB5[1] = NULL;


	ahHoldDown = NULL;
	ahPayload = NULL;
	iStructState = 0;

	//thRSRB5[0] = NULL;
	//thRSRB5[1] = NULL;
	pRSRB5[0] = new RSRB5(this);
	pRSRB5[1] = new RSRB5(this);
	fBlinkvert = false;
	fBlinkvertTimer = 0.0;
	fMETRunning = false;
	fPayloadMass = 0.0;
	fMET = -30.0;
	fTelemetryLog = false;


	for(int i = 0; i<4; i++)
	{
		
		//thSSME[i] = NULL;
		//thRetro1[i] = NULL;
		thRCS2[i] = NULL;
		thRCS2[i+4] = NULL;
		pSSME[i] = new SSME(this);
		
	}

	thJ2S = NULL;

	TgtRates = _V(0.0, 0.0, 0.0);
}

SDLV110::~SDLV110()
{
	delete pRSRB5[0];
	delete pRSRB5[1];
	for(int i = 0; i<4; i++)
	{
		
		delete pSSME[i];
	}
}

void SDLV110::AddFairingMesh(const VECTOR3& base)
{
	switch(fairing_type)
	{
	case FAIRING_NONE:
		break;
	case FAIRING_LONG_CLOSED:
		VECTOR3 X = base + _V(0.0, 0.0, 21.75);
		SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_LONGFAIRING_LEFT, &X), MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_LONGFAIRING_RIGHT, &X), MESHVIS_ALWAYS);
		break;
	}
}

int SDLV110::clbkConsumeBufferedKey(DWORD dwKey, bool fPressed, char* kstate)
{
	if(!fPressed)
	{
		if(!KEYMOD_SHIFT(kstate) && !KEYMOD_ALT(kstate) &&!KEYMOD_CONTROL(kstate))
		{
			switch(dwKey)
			{
			case OAPI_KEY_ADD:
				if(iStructState == 0)
				{
					StartIgnitionSequence();
					return 1;
				}
				return 0;
			case OAPI_KEY_J:
				{
					DeployPayload();
				}
				return 1;
			}
		}
		else {
		}
	}
	else 
	{
	}
	return 0;
}

void SDLV110::clbkDrawHUD(int iMode, const HUDPAINTSPEC* pHPS, HDC hdc)
{
	VESSEL2::clbkDrawHUD(iMode, pHPS, hdc);
	HUD_DrawVesselState(pHPS, hdc);
	HUD_DrawLast3Events(pHPS, hdc);
	
	switch(iStructState)
	{
	case 0:
	case 1:
		HUD_DrawSSMEState(pHPS, hdc);
		break;
	case 2:
		HUD_DrawStage2RCS(pHPS, hdc);
		break;
	}
	switch(iMode)
	{
	case HUD_SURFACE:
		HUD_DrawFlightDirector(pHPS, hdc);
		break;
	case HUD_NONE:
		break;
	}

}

void SDLV110::clbkLoadStateEx(FILEHANDLE scn, void* vs)
{
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "[%s:sdlv\\sirius] Load State...", GetName());
		fclose(pLog);
	}
	char* pcLine;
	while(oapiReadScenario_nextline(scn, pcLine))
	{
		if(!strnicmp(pcLine, "STRUCT_STATE", 12)) 
		{
			sscanf(pcLine + 12, "%d", &iStructState);
		} 
		else if(!strnicmp(pcLine, "FAIRING", 7))
		{
			ParseFairingState(pcLine);
		}
		else if(!strnicmp(pcLine, "PAYLOAD_MASS", 12))
		{
			float fX = SDLV110_MAXPAYLOADMASS;
			sscanf(pcLine + 12, "%f", &fX);
			fPayloadMass = fX;
		}
		else
			ParseScenarioLineEx(pcLine, vs);

	}

	switch(iStructState)
	{
	case 0:
		SetupState0();
		break;
	case 1:
		SetupState1();
		break;
	}
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done\n", GetName());
		fclose(pLog);
	}
}

void SDLV110::clbkSaveState(FILEHANDLE scn)
{
	SaveDefaultState(scn);
	oapiWriteScenario_int(scn, "STRUCT_STATE", iStructState);
	SaveFairingState(scn);
	if(fPayloadMass > 0.0)
	{
		oapiWriteScenario_float(scn, "PAYLOAD_MASS", fPayloadMass);
	}
}

void SDLV110::clbkSetClassCaps(FILEHANDLE cfg)
{
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "[%s:sdlv\\sirius] Set class caps ...", GetName());
		fclose(pLog);
	}
	SetupState0();
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done\n", GetName());
		fclose(pLog);
	}
}


void SDLV110::clbkSetStateEx(const void* status)
{
	DefSetStateEx(status);
}

void SDLV110::clbkPostCreation()
{
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "[%s:sdlv\\sirius] Post creation...", GetName());
		fclose(pLog);
	}
	 oapiGetHeading(GetHandle(), &fLaunchPadHeading);
	 {
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done\n", GetName());
		fclose(pLog);
	}
}

void SDLV110::clbkPostStep(double fSimT, double fDeltaT, double fMJD)
{
	if(oapiGetSysTime()> fBlinkvertTimer)
	{
		fBlinkvert = !fBlinkvert;
		fBlinkvertTimer = oapiGetSysTime() + 0.5;
	}

	

	if(fMETRunning)
	{
		fMET += fDeltaT;
	}

	if(iStructState < 1)
	{
		pRSRB5[0]->clbkPostStep(fSimT, fDeltaT);
		pRSRB5[1]->clbkPostStep(fSimT, fDeltaT);
	}

	if(iStructState < 2)
	{
		for(int i = 0; i<4; i++)
		{
			pSSME[i]->clbkPostStep(fSimT, fDeltaT);
		}
	}

	switch(iStructState)
	{
	case 0:
		{
			
			if(!pRSRB5[0]->IsIgnited() || !pRSRB5[1]->IsIgnited())
			{
				bool bSSMEReady = true;
				for(int i = 0; i<4; i++)
				{
					if(!pSSME[i]->IsGood() || pSSME[i]->GetChamberPressure() < 170.0)
					{
						bSSMEReady = false;
					}
				}
				if(bSSMEReady && fMET >= -0.3)
				{
					pRSRB5[0]->Ignite();
					pRSRB5[1]->Ignite();
					fMETRunning = true;
				} else if (fMET >0)
				{
					fMETRunning = false;
					fMET = 0.0;
				}
			}

			
			if(fMET > 132.0)
			{
				SeparateSRB();
				
			}
			else if(fMET > -5.0) 
			{
				for(int i = 0; i<4; i++)
				{
					pSSME[i]->SetThrustLevel(1.0);
				}
			} else if(fMET > -6.6)
			{
				for(int i = 0; i<4; i++)
				{
					if(pSSME[i]->IsGood())
						pSSME[i]->Ignite();
				}
			}

			

		}
		break;
	case 1:
		{
			if(GetPropellantMass(phFirst) < 6.0E3)
			{
				bool bShutdown = true;
				for(int i = 0; i<4; i++)
				{
					pSSME[i]->SetThrustLevel(0.0);
					bShutdown &= (pSSME[i]->GetChamberPressure() < 30.0 && pSSME[i]->IsGood());
				}
				if(bShutdown && fMET >= 350.0)
					SeparateFirstStage();
			}
		}
		break;
	case 2:
		{
			if(fMET < 355.0)
			{
				SetThrusterGroupLevel(THGROUP_ATT_FORWARD, 1.0);
			} else if(fMET < 360.0)
			{
				SetThrusterGroupLevel(THGROUP_ATT_FORWARD, 0.0);
			}
		}
		break;

	}

	double fTruePitch = GetPitch();
	if(fabs(GetBank()) > 90 * RAD)
		fTruePitch = 2 * 90 * RAD - fTruePitch;
	
	//In local horizon plane coordinates: x = N, y = UP, z = E
	VECTOR3 Tgt_O = _V(
		cos(GetTPitch(fMET)) * sin(GetTHeading(fDeltaT)), 
		sin(GetTPitch(fMET)), 
		cos(GetTPitch(fMET)) * cos(GetTHeading(fDeltaT)));
	//VECTOR3 Cur_O = _V(0.0, sin(GetPitch()),cos(GetPitch()));

	oapiGetHeading(GetHandle(), &fYaw);
	fPitch = GetPitch();
	fBank = GetBank();

	VECTOR3 Tgt = HorizonToLoc(Tgt_O);

	if(length(Tgt) == 0.0)
	{
		fPitchError = 0.0;
		fYawError = 0.0;
	}
	else
	{
		fPitchError = atan2(Tgt.y, Tgt.z);
		fYawError = atan2(Tgt.x, Tgt.z);
		
	}

	if(fMET < 7.5)
	{
		fBankError = 0.0;
	}
	else
	{
		fBankError = fBank;
	}
	
	/*
	sprintf(oapiDebugString(), "[%f,%f].[%f, %f, %f] ==> [%f, %f, %f] ==> [%f, %f, %f] ==> [%f, %f]", 
		GetTPitch(fMET) * DEG, GetTHeading(fDeltaT) * DEG,
		fPitch*DEG, fYaw*DEG, fBank*DEG,
		Tgt_O.x, Tgt_O.y, Tgt_O.z,
		Tgt.x, Tgt.y, Tgt.z, 
		fPitchError * DEG, fYawError * DEG);
		*/
	
	//fPitchError = (GetTPitch(fMET) - GetPitch()) * cos(GetBank());
	//fYawError = (GetTPitch(fMET) - GetPitch()) * sin(GetBank());
	
	
	TgtRates.x = PitchRateControl.Calc(fPitchError, fDeltaT);
	TgtRates.y = YawRateControl.Calc(fYawError, fDeltaT);
	TgtRates.z = BankRateControl.Calc(fBankError, fDeltaT);

	GetAngularVel(AttRate);
	//sprintf(oapiDebugString(), "[%f, %f, %f] (/s) // [%f, %f, %f]",
	//	TgtRates.x * DEG, TgtRates.y * DEG, TgtRates.z * DEG,
	//	AttRate.x*DEG, AttRate.y*DEG, AttRate.z * DEG);
	
	//At this point inhibit Yaw and Pitch rate command if possible >>>


	if(fMET > 227.0 
		&& GetDynPressure() < 9.0 
		&& fairing_type != FAIRING_NONE)
	{
		SeparateFairing();
	}

	DoTVC(fSimT, fDeltaT);
}

void SDLV110::CreateRSRB5()
{
	/*
	SURFHANDLE shSRBTex = oapiRegisterExhaustTexture("exhaust_atsrb");

	PARTICLESTREAMSPEC srb_contrail = {
		0, 12.0, 3, 150.0, 0.4, 8.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
	};
	PARTICLESTREAMSPEC srb_exhaust = {
		0, 3.0, 40, 250.0, 0.04, 0.7, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	if(!thRSRB5[0])
	{
		thRSRB5[0] = CreateThruster(_V(6.8, 0, -27.5), _V(0.0, 0.0, 1.0), RSRB5_THRUST0, 
				phRSRB5[0], RSRB5_ISP0, RSRB5_ISPSL);
	}

	if(!thRSRB5[1])
	{
		thRSRB5[1] = CreateThruster(_V(-6.8, 0, -27.5), _V(0.0, 0.0, 1.0), RSRB5_THRUST0, 
				phRSRB5[1], RSRB5_ISP0, RSRB5_ISPSL);
	}

	srb_exhaust.tex = oapiRegisterParticleTexture ("Contrail2");
	srb_contrail.tex = oapiRegisterParticleTexture ("Contrail4");

	for(int i = 0; i<2; i++)
	{
		VECTOR3 vX = _V(7.0, 0.0, -35.6);
		vX.x -= i * 14;
		AddExhaust(thRSRB5[i], 40.0, 3.8, 0, shSRBTex);
		AddExhaustStream(thRSRB5[i], vX, &srb_exhaust);
		vX.z -= 25.0;
		AddExhaustStream(thRSRB5[i], vX, &srb_contrail);
	}
	*/

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate SRBs ...");
		fclose(pLog);
	}
	pRSRB5[0]->Create(_V(-6.8, 0, -27.5), _V(0.0, 0.0, 1.0), phRSRB5[0]);
	pRSRB5[1]->Create(_V(6.8, 0, -27.5), _V(0.0, 0.0, 1.0), phRSRB5[1]);
}

void SDLV110::CreateSSMEs() 
{
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate SSME ...");
		fclose(pLog);
	}

	pSSME[0]->Create(_V(2.0, 2.0, -27.5), _V(0.0, 0.0, 1.0), phFirst);
	pSSME[1]->Create(_V(2.0, -2.0, -27.5), _V(0.0, 0.0, 1.0), phFirst);
	pSSME[2]->Create(_V(-2.0, -2.0, -27.5), _V(0.0, 0.0, 1.0), phFirst);
	pSSME[3]->Create(_V(-2.0, 2.0, -27.5), _V(0.0, 0.0, 1.0), phFirst);

	SetDefaultPropellantResource(phFirst);

}

void SDLV110::CreateStage2RCS()
{
	SURFHANDLE shRCSTex = oapiRegisterExhaustTexture("exhaust_atrcs");
	//Forward / Yaw engines
	for(int i = 0; i<8; i++)
	{
		if(thRCS2[i])
		{
			DelThruster(thRCS2[i]);
			thRCS2[i] = NULL;
		}
	}
	thRCS2[0] = CreateThruster(SDLV_STAGE2RCS_POS1, _V(0.0, 0.0, 1.0), SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);
	thRCS2[1] = CreateThruster(SDLV_STAGE2RCS_POS2, _V(0.0, 0.0, 1.0), SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);
	thRCS2[2] = CreateThruster(SDLV_STAGE2RCS_POS3, _V(0.0, 0.0, 1.0), SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);
	thRCS2[3] = CreateThruster(SDLV_STAGE2RCS_POS4, _V(0.0, 0.0, 1.0), SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);

	thRCS2[4] = CreateThruster(SDLV_STAGE2RCS_POS5, SDLV_STAGE2RCS_DIR5, SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);
	thRCS2[5] = CreateThruster(SDLV_STAGE2RCS_POS7, SDLV_STAGE2RCS_DIR7, SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);
	thRCS2[6] = CreateThruster(SDLV_STAGE2RCS_POS6, SDLV_STAGE2RCS_DIR6, SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);
	thRCS2[7] = CreateThruster(SDLV_STAGE2RCS_POS8, SDLV_STAGE2RCS_DIR8, SDLV_STAGE2RCS_THRUST0, phSecondRCS, 2200, 600);

	for(i = 0; i<8; i++)
	{
		AddExhaust(thRCS2[i], 3.0, 0.3, 0.2, shRCSTex);
	}

	CreateThrusterGroup(thRCS2, 4, THGROUP_ATT_FORWARD);

	CreateThrusterGroup(thRCS2, 2, THGROUP_ATT_YAWLEFT);

	CreateThrusterGroup(thRCS2 + 2, 2, THGROUP_ATT_YAWRIGHT);

	CreateThrusterGroup(thRCS2 + 4, 2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup(thRCS2 + 6, 2, THGROUP_ATT_PITCHDOWN);

	THRUSTER_HANDLE thTemp[2];
	thTemp[0] = thRCS2[4];
	thTemp[1] = thRCS2[7];

	CreateThrusterGroup(thTemp, 2, THGROUP_ATT_BANKRIGHT);
	thTemp[0] = thRCS2[5];
	thTemp[1] = thRCS2[6];
	CreateThrusterGroup(thTemp, 2, THGROUP_ATT_BANKLEFT);
}

void SDLV110::DeployPayload()
{
	if(GetAttachmentStatus(ahPayload))
	{
		DetachChild(ahPayload, 0.3);
		fPayloadMass = 0.0;
	}
}

void SDLV110::DoTVC(double fSimT, double fDeltaT)
{
	VECTOR3 RateError = TgtRates - AttRate;
	
	double fPitchTVC = PitchControl.Calc(RateError.x, fDeltaT);
	double fYawTVC = YawControl.Calc(RateError.y, fDeltaT);
	double fBankTVC = BankControl.Calc(RateError.z, fDeltaT);
	
	VECTOR3 BankDir = _V(0.0, 0.0, 0.0);
	VECTOR3 Mix;

	switch(iStructState)
	{
	case 0:
		{
			VECTOR3 TgtDir = _V(sin(fYawTVC * SDLV_SRB_TVC_SCALE), sin(-fPitchTVC * SDLV_SRB_TVC_SCALE), cos(fYawTVC * SDLV_SRB_TVC_SCALE) * cos(fPitchTVC * SDLV_SRB_TVC_SCALE));
			for(int i = 0; i<2; i++)
			{
				BankDir = _V(0, 0 , 0);
				VECTOR3 pos = pRSRB5[i]->GetPosition();
				pos.z = 0;
				double lpos = length(pos);
				if(lpos > 0)
				{
					pos = pos * (1.0/lpos);
					BankDir = _V(sin(fBankTVC) * pos.y, sin(fBankTVC) * -pos.x, cos(fBankTVC));
					
					Mix = (TgtDir+ BankDir) * 0.5;
					pRSRB5[i]->SetThrustDir(Mix);
				}
			}
		}
	case 1:
		{
			VECTOR3 TgtDir = _V(sin(fYawTVC), sin(-fPitchTVC), cos(fYawTVC) * cos(fPitchTVC));
			for (int i = 0; i<4; i++)
			{
				BankDir = _V(0, 0 , 0);
				VECTOR3 pos = pSSME[i]->GetPosition();
				pos.z = 0;
				double lpos = length(pos);
				if(lpos > 0)
				{
					pos = pos * (1.0/lpos);
					BankDir = _V(sin(fBankTVC) * pos.y, sin(fBankTVC) * -pos.x, cos(fBankTVC));
					
					Mix = (TgtDir+ BankDir) * 0.5;
					pSSME[i]->SetThrustDir(Mix);
				}
			}
		}
		break;
	case 2:
		break;
	}
}

double SDLV110::GetFairingMass() const
{
	switch(fairing_type)
	{
	case FAIRING_NONE:
		return 0.0;
	case FAIRING_LONG_CLOSED:
		return 2300.0;
	}
	return 0.0;
}

double SDLV110::GetPayloadMass() const
{
	return fPayloadMass;
}

double SDLV110::GetTHeading(double fDeltaT) const
{
	
	if(fMET < 7.5)
	{
		return fLaunchPadHeading;
	}
	else if(fMET < 15.0)
	{
		return fLaunchPadHeading + (90 * RAD - fLaunchPadHeading)/7.5 * (fMET - 7.5);
	}
	else
	{
		return 90.0 * RAD;
	}
	
}

double SDLV110::GetTPitch(double fMET) const
{
	if(fMET < 7.5)
	{
		return 90.0 * RAD;
		//return 00.0 * RAD;
	}
	else if(fMET < 10.0)
	{
		return 90.0 * RAD - (fMET - 7.5) * RAD;
	}
	else if(fMET < 15.0)
	{
		return 87.5 * RAD - 0.5 * RAD * (fMET-10.0);
	}
	else if(fMET < 80.0)
	{
		return 85.0 * RAD - 0.76923 * RAD * (fMET - 15.0);
	}
	else if(fMET < 125.0)
	{
		return 35.0 * RAD - 0.111111 * RAD * (fMET - 80.0);
	} 
	else if(fMET < 135.0)
	{
		return 30.0 * RAD;
	}
	else if(fMET < 235.0)
	{
		return 30.0 * RAD - 0.1 * RAD * (fMET - 135);
	}
	else if(fMET < 300.0)
	{
		return  20.0 * RAD - 0.07692 * RAD * (fMET - 235.0);
	}
	else if(fMET < 340.0)
	{
		return 15.0 * RAD + 0.250 * RAD * (fMET - 300.0);
	}
	else if(fMET < 360.0)
	{
		return 25.0 * RAD;
	}
	else 
		return 0.0 * RAD;
}

MATRIX3 CreateRotMat(const VECTOR3& axis, double angle)
{
	double cosa = cos(angle);
	double sina = sin(angle);
	MATRIX3 r;
	r.m11 = (1-cosa) * pow(axis.x, 2) + cosa;
	r.m12 = (1-cosa) * axis.x*axis.y + sina * -axis.z;
	r.m13 = (1-cosa) * axis.x*axis.z + sina * axis.y;
	r.m21 = (1-cosa) * axis.x * axis.y + axis.z * sina;
	r.m22 = (1-cosa) * axis.y*axis.y + cosa;
	r.m23 = (1-cosa) * axis.y*axis.z + sina * -axis.x;
	r.m31 = (1-cosa) * axis.x * axis.z - axis.y * sina;
	r.m32 = (1-cosa) * axis.y * axis.z + sina * axis.x;
	r.m33 = (1-cosa) * axis.z*axis.z + cosa;
	return r;
}

VECTOR3 MatMult(const MATRIX3& mat, const VECTOR3& vec)
{

	VECTOR3 prod;
	prod.x = vec.x * mat.m11 + vec.y * mat.m12 + vec.y * mat.m13;
	prod.y = vec.x * mat.m21 + vec.y * mat.m22 + vec.y * mat.m23;
	prod.z = vec.x * mat.m31 + vec.y * mat.m32 + vec.y * mat.m33;
	return prod;
}

MATRIX3 MatMult(const MATRIX3& mat_a, const MATRIX3& mat_b)
{

	MATRIX3 prod;
	for(int i = 0; i<3; i++)
	{
		for(int k = 0; k<3; k++)
		{
			prod.data[i*3 + k] = 0.0;
			for(int j=0; j<3; j++)
			{
				prod.data[i*3 + k] += mat_a.data[i*3 + j] * mat_b.data[j * 3 + k];
			}
		}
	}
	return prod;
}

VECTOR3 SDLV110::HorizonToLoc(const VECTOR3& horVec) const
{
	const VECTOR3 locX = _V(1.0, 0.0, 0.0);
	const VECTOR3 locY = _V(0.0, 1.0, 0.0);
	const VECTOR3 locZ = _V(0.0, 0.0, 1.0);
	
	VECTOR3 horX, horY, horZ;
	VECTOR3 locres;

	HorizonRot(locX, horX);
	HorizonRot(locY, horY);
	HorizonRot(locZ, horZ);

	locres.x = dotp(horX, horVec);
	locres.y = dotp(horY, horVec);
	locres.z = dotp(horZ, horVec);
	return locres;
}

void SDLV110::HUD_DrawFlightDirector(const HUDPAINTSPEC* pHPS, HDC hdc)
{
	int iX = pHPS->CX + (int)(fYawError * DEG * pHPS->Scale);
	int iY = pHPS->CY - (int)(fPitchError * DEG * pHPS->Scale);
	int iR = pHPS->Markersize / 2;
	Ellipse(hdc, iX - iR, iY - iR, iX + iR, iY + iR);
	//two lines for indicating Roll Error
	int iX1 = (int)(1.0 * pHPS->Markersize * cos(fBankError));
	int iX2 = (int)(0.5 * pHPS->Markersize * cos(fBankError));
	int iY1 = (int)(1.0 * pHPS->Markersize * sin(fBankError));
	int iY2 = (int)(0.5 * pHPS->Markersize * sin(fBankError));
	MoveToEx(hdc, iX + iX1, iY + iY1, NULL);
	LineTo(hdc, iX + iX2, iY + iY2);

	MoveToEx(hdc, iX - iX1, iY - iY1, NULL);
	LineTo(hdc, iX - iX2, iY - iY2);

	//Top Marker for indicating Roll Error
	iX1 = (int)(0.8 * pHPS->Markersize * sin(fBankError));
	iX2 = (int)(0.5 * pHPS->Markersize * sin(fBankError));
	iY1 = (int)(0.8 * pHPS->Markersize * cos(fBankError));
	iY2 = (int)(0.5 * pHPS->Markersize * cos(fBankError));
	MoveToEx(hdc, iX + iX1, iY - iY1, NULL);
	LineTo(hdc, iX + iX2, iY - iY2);
}

void SDLV110::HUD_DrawLast3Events(const HUDPAINTSPEC* pHPS, HDC hdc)
{
	char pcBuffer[300];
	int iBX = (int)(pHPS->W * 0.75) + 10;
	int iBW = (int)(pHPS->W * 0.20);
	int iCX = (iBX + iBW/2);
	int iBY = 40;
	int iBH = 60;

	Rectangle(hdc, iBX, iBY, iBX + iBW, iBY + iBH);
	
	sprintf(pcBuffer, "T%+7.2f", fMET);
	if(fMETRunning || fBlinkvert)
	{
		TextOut(hdc, iBX + 4, iBY, pcBuffer, 7);
	}

	//Find next event
	int i = 0;
	while(i<NEVENTS && MASTEREVENTTABLE[i].fMET < fMET)
	{
		i++;
	}

	Rectangle(hdc, iBX, iBY + 25, iBX + iBW, iBY + 37);


	if(i < NEVENTS)
	{
		int k = 10;
		for(int j = i-1; j<i+2 && j<NEVENTS; j++)
		{
			if(j >= 0)
			{
				bool fBlinkEvent = false;	
				if(MASTEREVENTTABLE[j].fMET < fMET + 3.0 && MASTEREVENTTABLE[j].fMET + 2.0 > fMET)
				{
					fBlinkEvent = true;
				}
				if(!fBlinkEvent || fBlinkvert)
				{
					sprintf(pcBuffer, "T%+7.2f   %s", MASTEREVENTTABLE[j].fMET, MASTEREVENTTABLE[j].pcEvent);
					TextOut(hdc, iBX + 4, iBY + k, pcBuffer, strlen(pcBuffer));
				}
			}
			k += 12;
		}
	}
	
}

void SDLV110::HUD_DrawSSMEState(const HUDPAINTSPEC* pHPS, HDC hdc)
{
	int iBX = 10;
	int iBW = (int)(pHPS->W * 0.20);
	int iCX = (iBX + iBW/2);
	int iBY = 150;
	int iBH = pHPS->CY - iBY;
	char pchBuffer[5];
	for(int i = 0; i<4; i++)
	{

		int iX = iBX + (iBW/4) * i;
		sprintf(pchBuffer, "%d", i+1);
		if(pSSME[i]->IsGood() || fBlinkvert)
		{
			Rectangle(hdc, iX, iBY + 120, iX + 10, iBY+130);
		}
		else
		{
			//Rectangle(hdc, iX, iBY + 120, iX + 10, iBY+120);
		}
		TextOut(hdc, iX, iBY, pchBuffer, 1);
		MoveToEx(hdc, iX, iBY+20, NULL);
		LineTo(hdc, iX+10, iBY + 20);
		LineTo(hdc, iX+10, iBY + 100);
		LineTo(hdc, iX, iBY + 100);
		MoveToEx(hdc, iX+5, iBY+60, NULL);
		LineTo(hdc, iX+10, iBY + 60);
		int iDY = iBY + 100 - (int)(80 * (pSSME[i]->GetChamberPressure() / 200.0));

		if(pSSME[i]->GetChamberPressure() > 60.0 || fBlinkvert)
		{
			MoveToEx(hdc, iX+10, iDY, NULL);
			LineTo(hdc, iX, iDY + 5);
			LineTo(hdc, iX, iDY - 5);
			LineTo(hdc, iX + 10, iDY);
		}
		iDY = iBY + 100 - (int)(80 * pSSME[i]->GetThrustCommand());
		MoveToEx(hdc, iX+12, iDY, NULL);
		LineTo(hdc, iX+15, iDY + 2);
		LineTo(hdc, iX+15, iDY - 2);
		LineTo(hdc, iX + 12, iDY);
		//Draw TVC gimbal state
		Rectangle(hdc, iX, iBY + 135, iX+20, iBY+ 155);
		double fX = pSSME[i]->GetXDeflection() / (4.0 * RAD);
		double fY = pSSME[i]->GetYDeflection() / (4.0 * RAD);
		bool fDrawMe = true;
		if(fX > 1.0)
		{
			fX = 1.0;
			fDrawMe = fBlinkvert;
		}
		if(fX < -1.0)
		{
			fX = -1.0;
			fDrawMe = fBlinkvert;
		}
		if(fY > 1.0)
		{
			fY = 1.0;
			fDrawMe = fBlinkvert;
		}
		if(fY < -1.0)
		{
			fY = -1.0;
			fDrawMe = fBlinkvert;
		}
		int icX = iX+10 + (int)(fX * 10);
		int icY = iBY+145 + (int)(fY * 10);
		if(fDrawMe)
			Ellipse(hdc, icX - 2, icY-2, icX + 2, icY + 2);
	}
}

void SDLV110::HUD_DrawStage2RCS(const HUDPAINTSPEC* pHPS, HDC hdc)
{
	//Draw RCS propellant state as % display 
	int iBX = 10;
	int iBW = (int)(pHPS->W * 0.20);
	int iCX = (iBX + iBW/2);
	int iBY = 150;
	int iBH = pHPS->CY - iBY;
	
	MoveToEx(hdc, iBX+30, iBY, NULL);
	LineTo(hdc, iBX + 30, iBY + 10);
	LineTo(hdc, iBW, iBY + 10);
	LineTo(hdc, iBW, iBY);

	int iFX = iBX + (int)((iBW - iBX) * (GetPropellantMass(phSecondRCS)/GetPropellantMaxMass(phSecondRCS)));
	MoveToEx(hdc, iFX, iBY + 10, NULL);
	LineTo(hdc, iFX + 5, iBY);
	LineTo(hdc, iFX - 5, iBY);
	LineTo(hdc, iFX, iBY+10);

}

/**
 * Draw in right top corner, below the information data
 */
void SDLV110::HUD_DrawVesselState(const HUDPAINTSPEC* pHPS, HDC hdc)
{
	int iBX = (int)(pHPS->W * 0.75);
	int iBW = (int)(pHPS->W * 0.20);
	int iCX = (iBX + iBW/2);
	int iBY = 105;
	int iBH = (pHPS->CY + 30) - iBY;

	
	//Make the full length of the display = 100m
	double fS = (1.0 * iBH) / 100.0; 
	//draw fairing
	if(fairing_type != FAIRING_NONE)
	{
		MoveToEx(hdc, iCX, iBY, NULL);
		LineTo(hdc, iCX-(int)(4.7 * fS), iBY+(int)(8.0 * fS));
		LineTo(hdc, iCX-(int)(4.7 * fS), iBY+(int)(34.0 * fS));
		LineTo(hdc, iCX+(int)(4.7 * fS), iBY+(int)(34.0 * fS));
		LineTo(hdc, iCX+(int)(4.7 * fS), iBY+(int)(8.0 * fS));
		LineTo(hdc, iCX, iBY);
		LineTo(hdc, iCX, iBY+(int)(34.0 * fS));
	}
	//Draw second stage
	{
		//Adapter
		MoveToEx(hdc, iCX-(int)(4.7 * fS), iBY+(int)(35.0 * fS), NULL);
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(35.0 * fS));
		LineTo(hdc, iCX+(int)(4.7 * fS), iBY+(int)(34.0 * fS));
		LineTo(hdc, iCX-(int)(4.7 * fS), iBY+(int)(34.0 * fS));
		LineTo(hdc, iCX-(int)(4.2 * fS), iBY+(int)(35.0 * fS));
		//Stage
		LineTo(hdc, iCX-(int)(4.2 * fS), iBY+(int)(40.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(40.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(35.0 * fS));
		//Aft Cone
		MoveToEx(hdc, iCX-(int)(4.2 * fS), iBY + (int)(40.0 * fS), NULL);
		LineTo(hdc, iCX-(int)(1.6 * fS), iBY+(int)(44.0 * fS));
		LineTo(hdc, iCX+(int)(1.6 * fS), iBY+(int)(44.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(40.0 * fS));
	}
	//Draw core
	if(iStructState<2)
	{
		//Adapter
		MoveToEx(hdc, iCX-(int)(4.2 * fS), iBY+(int)(40.0 * fS), NULL);
		LineTo(hdc, iCX-(int)(4.2 * fS), iBY+(int)(47.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(47.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(40.0 * fS));
		//Stage
		MoveToEx(hdc, iCX-(int)(4.2 * fS), iBY+(int)(47.0 * fS), NULL);
		LineTo(hdc, iCX-(int)(4.2 * fS), iBY+(int)(93.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(93.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(47.0 * fS));
		//
		MoveToEx(hdc, iCX-(int)(4.2 * fS), iBY+(int)(93.0 * fS), NULL);
		LineTo(hdc, iCX-(int)(3 * fS), iBY+(int)(98.0 * fS));
		LineTo(hdc, iCX+(int)(3 * fS), iBY+(int)(98.0 * fS));
		LineTo(hdc, iCX+(int)(4.2 * fS), iBY+(int)(93.0 * fS));
		
	}
	//draw boosters
	if(iStructState<1)
	{
		//Left
		MoveToEx(hdc, iCX-(int)(6.5 * fS), iBY+(int)(47.0 * fS), NULL);
		LineTo(hdc, iCX-(int)(8.35 * fS), iBY+(int)(50.0 * fS));
		LineTo(hdc, iCX-(int)(8.35 * fS), iBY+(int)(96.0 * fS));
		LineTo(hdc, iCX-(int)(9 * fS), iBY+(int)(99.0 * fS));
		LineTo(hdc, iCX-(int)(4 * fS), iBY+(int)(99.0 * fS));
		LineTo(hdc, iCX-(int)(4.65 * fS), iBY+(int)(96.0 * fS));
		LineTo(hdc, iCX-(int)(4.65 * fS), iBY+(int)(50.0 * fS));
		LineTo(hdc, iCX-(int)(6.5 * fS), iBY+(int)(47.0 * fS));
		//Right
		MoveToEx(hdc, iCX+(int)(6.5 * fS), iBY+(int)(47.0 * fS), NULL);
		LineTo(hdc, iCX+(int)(8.35 * fS), iBY+(int)(50.0 * fS));
		LineTo(hdc, iCX+(int)(8.35 * fS), iBY+(int)(96.0 * fS));
		LineTo(hdc, iCX+(int)(9 * fS), iBY+(int)(99.0 * fS));
		LineTo(hdc, iCX+(int)(4 * fS), iBY+(int)(99.0 * fS));
		LineTo(hdc, iCX+(int)(4.65 * fS), iBY+(int)(96.0 * fS));
		LineTo(hdc, iCX+(int)(4.65 * fS), iBY+(int)(50.0 * fS));
		LineTo(hdc, iCX+(int)(6.5 * fS), iBY+(int)(47.0 * fS));
	}

	
}

void SDLV110::ManagePayloadAttachment(VECTOR3 & ref)
{
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tDefine payload attachment ...");
		fclose(pLog);
	}
	if(!ahPayload)
	{
		ahPayload = CreateAttachment(false, ref, _V(0.0, 0.0, 1.0), _V(0.0, 1.0, 0.0), PAYLOAD_ATTACH_ID);
	} else
	{
		SetAttachmentParams(ahPayload, ref, _V(0.0, 0.0, 1.0), _V(0.0, 1.0, 0.0));
	}
}

void SDLV110::ParseFairingState(char* pchLine)
{
	char pchBuffer[500];
	sscanf(pchLine + 8, "%s", pchBuffer);
	if(!strnicmp(pchBuffer, "NONE", 4))
	{
		fairing_type = FAIRING_NONE;
	} else if(!strnicmp(pchBuffer, "LONG", 4))
	{
		fairing_type = FAIRING_LONG_CLOSED;
	}
}

void SDLV110::SaveFairingState(FILEHANDLE scn) const
{
	char pchBuffer[500];
	switch(fairing_type)
	{
	case FAIRING_NONE:
		sprintf(pchBuffer, "NONE");
		break;
	case FAIRING_LONG_CLOSED:
		sprintf(pchBuffer, "LONG");
		break;
	}
	oapiWriteScenario_string(scn, "FAIRING", pchBuffer);
}

void SDLV110::SeparateFairing()
{
	if(fairing_type == FAIRING_NONE)
		return;

	char pchName[400];
	VESSELSTATUS2 status_l, status_r;
	status_l.version = 2;
	status_l.flag = 0;
	status_r.version = 2;
	status_r.flag = 0;

	GetStatusEx(&status_l);	
	GetStatusEx(&status_r);	

	VECTOR3 RX = _V(0.0, 0.0, 0.0);

	switch(fairing_type)
	{
	case FAIRING_LONG_CLOSED:
		RX = _V(0.0, 0.0, 21.75);
		break;

	}
	fairing_type = FAIRING_NONE;
	
	switch(iStructState)
	{
		case 0:
			SetupState0();
			RX.z += 35.29;
			break;
		case 1:
			RX.z += 35.29;
			SetupState1();
			break;
		case 2:
			RX.z += 0.5;
			SetupState2();
			break;
	}
	sprintf(pchName, "%s - Left Fairing", GetName());
	VECTOR3 dv, dp;
	GlobalRot(_V(-3.796, 0.0, 0.0), dv);
	GlobalRot(RX, dp);
	status_l.rpos += dp;
	status_l.rvel += dv;
	status_l.vrot.y -= 10.0 * RAD;
	oapiCreateVesselEx(pchName, "sdlv/sdlv_fair_ll", &status_l);

	sprintf(pchName, "%s - right Fairing", GetName());

	status_r.rpos += dp;
	status_r.rvel -= dv;
	status_r.vrot.y += 10.0 * RAD;

	oapiCreateVesselEx(pchName, "sdlv/sdlv_fair_lr", &status_r);

}

void SDLV110::SeparateFirstStage()
{
	VESSELSTATUS2 status;
	char pchName[400];
	status.version = 2;
	status.flag = 0;
	GetStatusEx(&status);
	ShiftCentreOfMass(SDLV_STATE1_STATE2_COGSHIFT);
	sprintf(pchName, "%s-Core stage", this->GetName());
	oapiCreateVesselEx(pchName, "sdlv/sdlv_core", &status);
	//Reset control loops.
	SetupState2();
}

void SDLV110::SeparateSRB()
{
	VESSELSTATUS2 statel, stater;
	char pchName[400];
	statel.version = 2;
	statel.flag = 0;
	this->GetStatusEx(&statel);
	stater = statel;
	
	VECTOR3 offset, rvel;
	GlobalRot(SDLV_LEFTBOOSTER_OFFSET, offset);
	GlobalRot(_V(-0.2, 0.0, 0.0), rvel);
	
	statel.rpos += offset;
	statel.rvel += rvel;

	GlobalRot(SDLV_RIGHTBOOSTER_OFFSET, offset);
	GlobalRot(_V( 0.2, 0.0, 0.0), rvel);
	
	stater.rpos += offset;
	stater.rvel += rvel;

	sprintf(pchName, "%s-Left SRB", this->GetName());

	OBJHANDLE hVesselL = oapiCreateVesselEx(pchName, "sdlv\\RSRB5L", &statel);

	sprintf(pchName, "%s-Right SRB", this->GetName());

	OBJHANDLE hVesselR = oapiCreateVesselEx(pchName, "sdlv\\RSRB5R", &stater);

	SDLV_RSRB5* pV = static_cast<SDLV_RSRB5*>(oapiGetVesselInterface(hVesselL));
	pV->SetRemainingBurnTime(fMET);
	pV->SetRemainingPropellant(GetPropellantMass(phRSRB5[0]));
	pV = static_cast<SDLV_RSRB5*>(oapiGetVesselInterface(hVesselR));
	pV->SetRemainingBurnTime(fMET);
	pV->SetRemainingPropellant(GetPropellantMass(phRSRB5[1]));
	//Reset Control loops.
	BankControl.Reset();
	PitchControl.Reset();
	YawControl.Reset();
	
	SetupState1();
}

void SDLV110::SetupState0()
{
	SetSize(45.0);
	SetCameraOffset(_V(0.0, 5, 18.0));
	//SetCameraRotationRange(-PI, PI, 0.0, 0.0);
	SetCameraDefaultDirection(_V(0.0, 0.0, 1.0));
	
	SetEmptyMass(SDLV_STATE0_EMPTYMASS + GetFairingMass() + GetPayloadMass());
	SetTouchdownPoints(_V(0.0, -4.5, -55), _V(-3.8, 2.25, -55), _V( 3.8, 2.25, -55));

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "\n\tCreate Visual ...");
		fclose(pLog);
	}
	ClearMeshes();
	SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_FIRSTSTAGE), MESHVIS_ALWAYS);
	VECTOR3 vX = _V(0.0, 0.0, 29.9);
	SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_SECONDSTAGE, &vX), MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_RSRB5_LEFT, &SDLV_LEFTBOOSTER_OFFSET), MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_RSRB5_RIGHT, &SDLV_RIGHTBOOSTER_OFFSET), MESHVIS_ALWAYS);
	AddFairingMesh(_V(0.0, 0.0, 35.29));
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate SRB propellants ...");
		fclose(pLog);
	}

	if(!phRSRB5[0]) 
	{
		phRSRB5[0] = CreatePropellantResource(SDLV_RSRB5_PROPELLANTMASS);
	}

	if(!phRSRB5[1]) 
	{
		phRSRB5[1] = CreatePropellantResource(SDLV_RSRB5_PROPELLANTMASS);
	}
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate core propellants ...");
		fclose(pLog);
	}
	
	if(!phFirst) 
	{
		phFirst = CreatePropellantResource(SDLV_STAGE1_PROPELLANTMASS);
	}

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate second stage  propellants ...");
		fclose(pLog);
	}
	if(!phSecond) 
	{
		phSecond = CreatePropellantResource(SDLV_STAGE2_PROPELLANTMASS);
	}
	if(!phSecondRCS) 
	{
		phSecondRCS = CreatePropellantResource(SDLV_STAGE2RCS_PROPELLANTMASS);
	}

	CreateRSRB5();
	CreateSSMEs();
	ManagePayloadAttachment(_V(0.0, 0.0, 35.29));

	
	iStructState = 0;
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\nCreation finished.\n");
		fclose(pLog);
	}
}

void SDLV110::SetupState1()
{
	SetSize(45.0);
	SetEmptyMass(SDLV_STATE1_EMPTYMASS + GetFairingMass() + GetPayloadMass());
	SetTouchdownPoints(_V(0.0, -4.5, -55), _V(-3.8, 2.25, -55), _V( 3.8, 2.25, -55));
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "\n\tCreate Visual ...");
		fclose(pLog);
	}
	ClearMeshes();
	SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_FIRSTSTAGE), MESHVIS_ALWAYS);
	VECTOR3 vX = _V(0.0, 0.0, 29.9);
	SetMeshVisibilityMode(AddMesh(MESHNAME_SDLV_SECONDSTAGE, &vX), MESHVIS_ALWAYS);
	AddFairingMesh(_V(0.0, 0.0, 35.29));

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tDelete SRBs ...");
		fclose(pLog);
	}

	pRSRB5[0]->Destroy();
	pRSRB5[1]->Destroy();

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tDelete SRB propellants ...");
		fclose(pLog);
	}

	if(phRSRB5[0]) 
	{
		DelPropellantResource(phRSRB5[0]);
		phRSRB5[0] = NULL;
	}

	if(phRSRB5[1]) 
	{
		DelPropellantResource(phRSRB5[1]);
		phRSRB5[1] = NULL;
	}

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate core stage  propellants ...");
		fclose(pLog);
	}

	if(!phFirst) 
	{
		phFirst = CreatePropellantResource(SDLV_STAGE1_PROPELLANTMASS);
	}

	

	if(!phSecond) 
	{
		{
			FILE* pLog = fopen("orbiter.log", "at");
			fprintf(pLog, "done.\n\tCreate second stage  propellants ...");
			fclose(pLog);
		}
		phSecond = CreatePropellantResource(SDLV_STAGE2_PROPELLANTMASS);
	}
	if(!phSecondRCS) 
	{
		phSecondRCS = CreatePropellantResource(SDLV_STAGE2RCS_PROPELLANTMASS);
	}

	CreateSSMEs();
	ManagePayloadAttachment(_V(0.0, 0.0, 35.29));
	iStructState = 1;
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\nCreation finished.\n");
		fclose(pLog);
	}
}

void SDLV110::SetupState2()
{

	SetSize(11.0);
	SetEmptyMass(SDLV_STATE2_EMPTYMASS + GetFairingMass() + GetPayloadMass());
	SetTouchdownPoints(_V(0.0, -4.5, -5.5), _V(-3.8, 2.25, -5.5), _V( 3.8, 2.25, -5.5));
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "\n\tCreate visual ...");
		fclose(pLog);
	}
	ClearMeshes();
	AddMesh(MESHNAME_SDLV_SECONDSTAGE, &SDLV_STAGE2_STATE2_POS);
	AddFairingMesh(_V(0.0, 0.0, 0.5));
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tRegister  exhaust texture...");
		fclose(pLog);
	}
	SURFHANDLE shSSMETex = oapiRegisterExhaustTexture("exhaust_atsme");

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tDelete SRB propellants ...");
		fclose(pLog);
	}

	if(phRSRB5[0]) 
	{
		DelPropellantResource(phRSRB5[0]);
		phRSRB5[0] = NULL;
	}

	if(phRSRB5[1]) 
	{
		DelPropellantResource(phRSRB5[1]);
		phRSRB5[1] = NULL;
	}

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tDelete SSMEs ...");
		fclose(pLog);
	}

	for(int i = 0; i<4; i++)
	{
		pSSME[i]->Destroy();
	}

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tDelete core stage  propellants ...");
		fclose(pLog);
	}

	if(phFirst) 
	{
		DelPropellantResource(phFirst);
		phFirst = NULL;
	}

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate second stage propellants ...");
		fclose(pLog);
	}

	if(!phSecond) 
	{
		phSecond = CreatePropellantResource(SDLV_STAGE2_PROPELLANTMASS);
	}
	if(!phSecondRCS) 
	{
		phSecondRCS = CreatePropellantResource(SDLV_STAGE2RCS_PROPELLANTMASS);
	}

	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate J2S ...");
		fclose(pLog);
	}

	if(!thJ2S)
	{
		thJ2S = CreateThruster(_V(0.0, 0.0, -3), _V(0.0, 0.0, 1.0), J2S_THRUST0, 
			phSecond, J2S_ISP0, J2S_ISPSL);
		AddExhaust(thJ2S, 30.0, 1.0, 1.4, shSSMETex);
		CreateThrusterGroup(&thJ2S, 1, THGROUP_MAIN);
	}
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\n\tCreate RCS ...");
		fclose(pLog);
	}
	CreateStage2RCS();
	
	SetDefaultPropellantResource(phSecond);
	
	ManagePayloadAttachment(_V(0.0, 0.0, 0.5));
	iStructState = 2;
	{
		FILE* pLog = fopen("orbiter.log", "at");
		fprintf(pLog, "done.\nCreation finished.\n");
		fclose(pLog);
	}
}

void SDLV110::StartIgnitionSequence()
{
	fMET = -6.6;
	fMETRunning = true;
	for(int i = 0; i<4; i++)
	{
		pSSME[i]->Ignite();
	}
}


DLLCLBK VESSEL* ovcInit(OBJHANDLE hVessel, int iFlightModel)
{
	return new SDLV110(hVessel, iFlightModel);
}

DLLCLBK void ovcExit(VESSEL* pVessel)
{
	delete static_cast<SDLV110*>(pVessel);
}
