#if !defined(___ORBITER_ADDON_URWUMPES_SDLV110_LAUNCHER_H___)
#define ___ORBITER_ADDON_URWUMPES_SDLV110_LAUNCHER_H___

#include "OrbiterAPI.h"
#include "VesselAPI.h"
#include "SDLV_Engines.h"
#include "RSRB5.h"

static const char* MESHNAME_SDLV_FIRSTSTAGE = "sdlv/sdlv_stage1";
static const char* MESHNAME_SDLV_SECONDSTAGE = "sdlv/sdlv_stage2";


static const char* MESHNAME_SDLV_LONGFAIRING_LEFT = "sdlv/sdlv_fair_ll";
static const char* MESHNAME_SDLV_LONGFAIRING_RIGHT = "sdlv/sdlv_fair_lr";

static const char* PAYLOAD_ATTACH_ID = "SDLVPL";

const VECTOR3 SDLV_LEFTBOOSTER_OFFSET = _V(-6.50, 0.0, -2.52);
const VECTOR3 SDLV_RIGHTBOOSTER_OFFSET = _V(6.50, 0.0, -2.52);

const double SDLV_STAGE2RCS_THRUST0 = 2000.0;


const double SDLV_STAGE1_EMPTYMASS = 54.0E3;
const double SDLV_STAGE1_PROPELLANTMASS = 721.0E3;
const double SDLV_STAGE2_EMPTYMASS = 9069.0;
const double SDLV_STAGE2_PROPELLANTMASS = 105.000E3;
const double SDLV_STAGE2RCS_PROPELLANTMASS = 250;

const double SDLV110_MAXPAYLOADMASS = 110.0E3;

const double SDLV_STATE0_EMPTYMASS = 2*SDLV_RSRB5_EMPTYMASS + SDLV_STAGE1_EMPTYMASS + SDLV_STAGE2_EMPTYMASS;
const double SDLV_STATE1_EMPTYMASS = SDLV_STAGE1_EMPTYMASS + SDLV_STAGE2_EMPTYMASS;
const double SDLV_STATE2_EMPTYMASS = SDLV_STAGE2_EMPTYMASS;

const VECTOR3 SDLV_STAGE2_STATE2_POS = _V(0.0, 0.0, -5.5);
const VECTOR3 SDLV_STATE1_STATE2_COGSHIFT = _V(0.0, 0.0, 34.9);

const VECTOR3 SDLV_STAGE2RCS_POS1 = _V(3.384, 0.1, -8.75);
const VECTOR3 SDLV_STAGE2RCS_POS2 = _V(3.384, -0.1, -8.75);
const VECTOR3 SDLV_STAGE2RCS_POS3 = _V(-3.384, 0.1, -8.75);
const VECTOR3 SDLV_STAGE2RCS_POS4 = _V(-3.384, -0.1, -8.75);

const VECTOR3 SDLV_STAGE2RCS_POS5 = _V(3.4, 0.4, -7.5);
const VECTOR3 SDLV_STAGE2RCS_DIR5 = _V(0, -1.0, 0);
const VECTOR3 SDLV_STAGE2RCS_POS6 = _V(3.4, -0.4, -7.5);
const VECTOR3 SDLV_STAGE2RCS_DIR6 = _V(0,  1.0, 0);
const VECTOR3 SDLV_STAGE2RCS_POS7 = _V(-3.4, 0.4, -7.5);
const VECTOR3 SDLV_STAGE2RCS_DIR7 = _V(0,  -1.0, 0);
const VECTOR3 SDLV_STAGE2RCS_POS8 = _V(-3.4, -0.4, -7.5);
const VECTOR3 SDLV_STAGE2RCS_DIR8 = _V(0,  1.0, 0);

const double SDLV_SRB_TVC_SCALE = 0.5;

typedef struct {
	double fMET;
	char* pcEvent;
} EVENTSTRING;


const int NEVENTS = 15;

const EVENTSTRING MASTEREVENTTABLE[NEVENTS] = {
	{-6.6, "SSME IGN"},
	{0.0, "RSRB IGN"},
	{0.0, "LIFTOFF"},
	{7.5, "ROLL MVR"},
	{10.0, "PITCH MVR"},

	{55.0, "MAX-Q"},
	{132.0, "SRB SEP"},
	{227.0, "PLF SEP"},
	{345.0, "MECO"},
	{350.0, "STAGE 1 SEP"},

	{360.0, "J2S IGN1"},
	{720.0, "SSECO1"},
	{3420.0, "J2S IGN2"},
	{3432.0, "SSECO2"},
	{3500.0, "STAGE 2 SEP"}
};






class PIDLOOP
{
	double fKPR;
	double fTNR;
	double fTVR;
	double fErrorOld;
	double fErrorInt;
public:
	PIDLOOP(double fKPR, double fTNR, double fTVR);
	void Reset();
	double Calc(double fError, double fDeltaT);
};



/**
 * @todo Failure simulation class, containing a MET and a failure code, which gets triggered at this moment.
 * @todo Payload handling
 * @todo SRB separation
 * @todo First stage separation
 * @todo Fairing separation
 * @todo Second stage deorbit after payload deployment.
 * @todo How does the first stage TVC work? current APU or is there an upgrade planned?
 */
class SDLV110: public VESSEL2 {
public:
	typedef enum {
		FAIRING_NONE = 0,
		FAIRING_SHORT_OPEN,
		FAIRING_LONG_OPEN,
		FAIRING_SHORT_CLOSED,
		FAIRING_LONG_CLOSED
	} FAIRING_TYPE;
protected:
	bool fBlinkvert;
	double fBlinkvertTimer;

	

	PROPELLANT_HANDLE phRSRB5[2];
	PROPELLANT_HANDLE phFirst;
	
	PROPELLANT_HANDLE phSecond;
	PROPELLANT_HANDLE phSecondRCS;

	//THRUSTER_HANDLE thRSRB5[2];
	//THRUSTER_HANDLE thSSME[4];
	SSME* pSSME[4];
	RSRB5* pRSRB5[2];
	//THRUSTER_HANDLE thRetro1[4];
	THRUSTER_HANDLE thJ2S;
	THRUSTER_HANDLE thRCS2[8];
	/**
	 * Structural configuration of the launcher
	 * 0: Prelaunch State; RSRB5 + FirstStage + Second Stage + payload + fairing
	 * 1: first Stage + second Stage + payload + fairing
	 * 2: second stage + payload + fairing
	 */
	int iStructState;
	/**
	 * Type of the currently used fairing.
	 */
	FAIRING_TYPE fairing_type;
	/**
	 * Flag for the MET counter state.
	 * true = running
	 * false = stopped/halted
	 */
	bool fMETRunning;
	/**
	 * The current mission elapsed time, with MET zero being the time of liftoff.
	 */
	double fMET;
	/**
	 * The current attitude rates.
	 */
	VECTOR3 AttRate;
	/**
	 *  The targeted attitude rates.
	 */
	VECTOR3 TgtRates;

	/**
	 * The heading on the launch pad.
	 * Is needed for roll program as initial reference.
	 */
	double fLaunchPadHeading;

	double fYaw;
	double fPitch;
	double fBank;

	double fPitchError;
	double fYawError;
	double fBankError;

	
	PIDLOOP PitchRateControl;
	PIDLOOP BankRateControl;
	PIDLOOP YawRateControl;
	PIDLOOP PitchControl;
	PIDLOOP YawControl;
	PIDLOOP BankControl;

	//Holddown post
	ATTACHMENTHANDLE ahHoldDown;

	//Payload management;
	ATTACHMENTHANDLE ahPayload;
	double fPayloadMass;

	bool fTelemetryLog;
	char pchTLMLogName[400];
protected:
	void AddFairingMesh(const VECTOR3& base);
	/**
	 * Get the target pitch for the current phase of flight.
	 * Until SRB separation, this is a simple pitch by MET function, 
	 * after the SRB Separation, the pitch guidance switches to a orbit insertion mode, 
	 * aiming for an immediate orbit at MECO, a cruise orbit at SSECO1 and the 
	 * final orbit at SSECO2.
	 */
	double GetTPitch(double fMET) const;
	/**
	 * Get the target heading for the current flight phase. This is, the heading on a great circle to 
	 * reach a desired inclination. The azimuth ranges for launch are selected and optimized for KSC, 
	 * for implementing new launch sites, this function would need to be extended.
	 * 
	 */
	double GetTHeading(double fDeltaT) const;
	/**
	 * Draw a schematic of the vehicle into the upper right corner.
	 * indicate failures there.
	 */
	void HUD_DrawVesselState(const HUDPAINTSPEC* pHPS, HDC hdc);
	/**
	 * Draw a icon to guide the vessel to its flight target
	 */
	void HUD_DrawFlightDirector(const HUDPAINTSPEC* pHPS, HDC hdc);
	/**
	 * Draw the last event, the next following event and the second next event into the HUD
	 * as well as the MET timer.
	 */
	void HUD_DrawLast3Events(const HUDPAINTSPEC* pHPS, HDC hdc);

	/**
	 * Draw at the left side four indicators: 4x chamber pressure, 4x Good, 4x TVC State
	 */
	void HUD_DrawSSMEState(const HUDPAINTSPEC* pHPS, HDC hdc);
	/**
	 * Draw the status of the Stage 2 RCS : propellant remaining, tank pressures, thruster state.
	 */
	void HUD_DrawStage2RCS(const HUDPAINTSPEC* pHPS, HDC hdc);
	void SeparateFairing();
	void SeparateSRB();
	void SeparateFirstStage();
	void SetupState0();
	void SetupState1();
	void SetupState2();
	void StartIgnitionSequence();
	void DoTVC(double fSimT, double fDeltaT);
	void CreateJ2S();
	void CreateRSRB5();
	void CreateSSMEs();
	void CreateStage2RCS();
	double GetFairingMass() const;
	double GetPayloadMass() const;
	void ParseFairingState(char* pchLine);
	void SaveFairingState(FILEHANDLE scn) const;
	
	void ManagePayloadAttachment(VECTOR3 & ref);
	void DeployPayload();
	VECTOR3 HorizonToLoc(const VECTOR3& horVec) const;
public:
	SDLV110(OBJHANDLE hVessel, int iFlightModel); 
	virtual ~SDLV110();
	virtual int clbkConsumeBufferedKey(DWORD dwKey, bool fPressed, char* kstate);
	virtual void clbkLoadStateEx(FILEHANDLE scn, void* vs);
	virtual void clbkPostCreation();
	virtual void clbkPostStep(double fSimT, double fDeltaT, double fMJD);
	virtual void clbkSaveState(FILEHANDLE scn);
	virtual void clbkSetClassCaps(FILEHANDLE cfg);
	virtual void clbkSetStateEx(const void* status);
	virtual void clbkDrawHUD(int iMode, const HUDPAINTSPEC* pHPS, HDC hdc);
};

#endif
