#ifndef ___SDLV_ENGINES_H_INCLUDED___
#define ___SDLV_ENGINES_H_INCLUDED___

#include "OrbiterAPI.H"

const double SSME_THRUST0 = 2.278E6;
const double SSME_ISP0 = 4443.93;
const double SSME_ISPSL = 3561.03;

const double J2S_THRUST0 = 1.139E6;
const double J2S_ISP0 = 4179.06;
const double J2S_ISPSL = 1962.0;
//16 million newtons sea-level thrust = 17.437E6 N Vacuum
const double RSRB5_THRUST0 = 16.00E6;
const double RSRB5_ISP0 = 2308.0;
const double RSRB5_ISPSL = 2118.0;


class BASICENGINE {
	bool fRealized;
	double MAX_TVC;
	double MAX_TVC_RATE;
	double TURBINE_INERTIA;
	int MAX_IGNITIONS;

protected:
	VESSEL* pParent;
	VECTOR3 dir0;
	VECTOR3 pos0;
	double fPitch0;
	double fYaw0;
	double fPitch;
	double fYaw;
	double fPitchCmd;
	double fYawCmd;
	bool fIgnited;
	double fThrustCmd;
	int iIgniterCount;
	bool fIsIgniting;
	void SetMaximumNumberOfIgniters(int iMax);
	void SetNeutralThrustVector(VECTOR3& vtn);
	void SetTVCLimits(double fMaxTVC, double fMaxTVCRate);
	virtual void Realize();
	virtual void Unrealize();
	void FireIgniter();
	void DoTVC(double fSimT, double fDeltaT);
	VECTOR3 GetThrustVector() const;
	virtual void CalcEngineParameters(double fSimT, double fDeltaT);
	virtual bool CheckIgnitionConditions() const;
	virtual void DoEngineController(double fSimT, double fDeltaT);
	bool IgniterFired() const;
public:
	BASICENGINE(VESSEL* pVessel);
	virtual ~BASICENGINE();
	virtual void Create(VECTOR3& pos, VECTOR3& dir0, PROPELLANT_HANDLE phSource);
	virtual void Destroy();
	void SetThrustLevel(double fThrustLevel);
	double GetThrustCommand() const;
	virtual void Ignite();

	virtual void clbkPostStep(double fSimT, double fDeltaT);
	void SetThrustDir(VECTOR3 newDir);
	double GetXDeflection() const;
	double GetYDeflection() const;
	VECTOR3 GetPosition() const;
	bool IsRealized() const;
	bool IsIgnited() const;
};

class SSME : public BASICENGINE {
	typedef enum{
		MODE_IDLE,
		MODE_START,
		MODE_IGNITION,
		MODE_RUNNING,
		MODE_SHUTDOWN
	} SSME_MODE;

	SSME_MODE mode;
	double fTimer1;
	double fTimer2;
	double fThrustCmd2;

	bool fIsGood;
	
	THRUSTER_HANDLE thMe;
protected:
	virtual void CalcEngineParameters(double fSimT, double fDeltaT);
	virtual bool CheckIgnitionConditions() const;
	virtual void DoEngineController(double fSimT, double fDeltaT);
public:
	SSME(VESSEL *pVessel);
	virtual ~SSME();
	virtual void Create(VECTOR3& pos, VECTOR3& dir0, PROPELLANT_HANDLE phSource);
	virtual void Destroy();
	double GetChamberPressure() const;	
	void Ignite();
	virtual bool IsGood() const;
};

class J2S : public BASICENGINE {
	typedef enum{
		MODE_IDLE,
		MODE_IGNITION,
		MODE_RUNNING,
		MODE_SHUTDOWN
	} J2S_MODE;

	J2S_MODE mode;
	double fTimer1;
	bool fIsGood;
	THRUSTER_HANDLE thMe;
protected:
	virtual void CalcEngineParameters(double fSimT, double fDeltaT);
	virtual bool CheckIgnitionConditions() const;
	virtual void DoEngineController(double fSimT, double fDeltaT);
public:
	J2S(VESSEL *pVessel);
	virtual ~J2S();
	virtual void Create(VECTOR3& pos, VECTOR3& dir0, PROPELLANT_HANDLE phSource);
	virtual void Destroy();
	double GetChamberPressure() const;	
	void Ignite();
	virtual bool IsGood() const;
	
	
	
};

class RSRB5: public BASICENGINE {
	double fBurnTime;
	THRUSTER_HANDLE thMe;
	PROPELLANT_HANDLE phSRB;
	double fNextLogTime;
protected:
	virtual bool CheckIgnitionConditions() const;
	void CalcEngineParameters(double fSimT, double fDeltaT);
public:
	RSRB5(VESSEL *pVessel);
	virtual ~RSRB5();
	virtual void Create(VECTOR3& pos, VECTOR3& dir0, PROPELLANT_HANDLE phSource);
	virtual void Destroy();
	virtual void Ignite();
	virtual void SetRemainingBurnTime( double fBT);
	double GetChamberPressure() const;
};

#endif