#include "RSRB5.h"

void SRB_LiftFunction(double fAOA, double fMach, double fRe, double *pfCl, double* pfCm, double *pfCd)
{

	double fLatDrag = 0.45;
	double fLongDrag = 0.11;
	if(fMach > 1.0)
	{
		fLatDrag = 0.45;
		fLongDrag = 0.09;
	}
	*pfCl = sin(2 * fAOA) * fLatDrag;
	*pfCd = cos(fAOA) * fLongDrag + sin(fAOA) * fLatDrag;
	*pfCm = 0.1 * (-sin(3 * fAOA) + sin(fAOA)) * pow(cos(0.5 * fAOA), 2);

}

SDLV_RSRB5::SDLV_RSRB5(OBJHANDLE hVessel, int iFlightModel)
: VESSEL2(hVessel, iFlightModel)
{
	fIsLeft = false;
	pSRB = new RSRB5(this);
	iState = 0;
	fTimer = 0.0;
	phSRB = NULL;
	phBSM1 = NULL;
	phBSM2 = NULL;
}

SDLV_RSRB5::~SDLV_RSRB5()
{
	delete pSRB;
}

void SDLV_RSRB5::clbkLoadStateEx(FILEHANDLE scn, void* vs)
{
	char* line;
	while(oapiReadScenario_nextline(scn, line))
	{

		ParseScenarioLineEx(line, vs);
	}
	SetupPostSep();
}

void SDLV_RSRB5::clbkSaveState(FILEHANDLE scn)
{
	SaveDefaultState(scn);
	oapiWriteScenario_int(scn, "STATE", iState);
	if(iState == 0)
	{
		oapiWriteScenario_float(scn, "TIMER", fTimer);
	}
}

void SDLV_RSRB5::clbkSetClassCaps(FILEHANDLE cfg)
{
	char* line;
	while(oapiReadScenario_nextline(cfg, line))
	{

		if(!strnicmp(line, "MODEL_RIGHT", 11))
		{
			fIsLeft = false;
		}
		else if(!strnicmp(line, "MODEL_LEFT", 10))
		{
			fIsLeft = true;
		}
	}
	SetupPostSep();
}

void SDLV_RSRB5::clbkSetStateEx(const void* status)
{
	DefSetStateEx(status);
}

void SDLV_RSRB5::clbkPostStep(double fSimT, double fDeltaT, double fMJD)
{
	pSRB->clbkPostStep(fSimT, fDeltaT);
	switch(iState)
	{
	case 0:
		
		
		IgniteGroup1();
		IgniteGroup2();
		
		//When Group 2 burn finished)
		if(GetPropellantMass(phBSM2) == 0.0)
		{
			iState = 1;
		}
		break;
	case 1:
		//Coast until pressure switch arms.
		break;
	case 2:
		//Coast until baroswitch deploys pilot chute
		break;
	case 3:
		//Until baroswitch deploys drogue
		break;
	}
}

void SDLV_RSRB5::CreateBSMThrusters()
{
	PARTICLESTREAMSPEC bsm_exhaust = {
		0, 1.0, 40, 75.0, 0.04, 0.5, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	bsm_exhaust.tex = oapiRegisterParticleTexture ("Contrail2");
	
	//First group (0 - 3) 0,1 Forward 2,3 aft
	if(fIsLeft)
	{
		//_V(0.632, 1.738, 23.0)
		//First group (0 - 3) 0,1 Forward 2,3 aft
		thBSM[0] = CreateThruster(_V(0.632, 1.738, 23.0), _V(-0.296, -0.813, -0.5), BSM_THRUST0, phBSM1, 2000);
		thBSM[1] = CreateThruster(_V(0.632, 1.738, 23.0), _V(-0.296, -0.813, -0.5), BSM_THRUST0, phBSM1, 2000);
		thBSM[2] = CreateThruster(_V(0.632, 1.738, -23.0), _V(-0.242, -0.696, 0.707), BSM_THRUST0, phBSM1, 2000);
		thBSM[3] = CreateThruster(_V(0.632, 1.738, -23.0), _V(-0.242, -0.696, 0.707), BSM_THRUST0, phBSM1, 2000);
		
		//second group (4 - 7) 4,5 Forward 6,7 aft
		thBSM[4] = CreateThruster(_V(0.602, 1.708, 23.3), _V(-0.296, -0.696, -0.5), BSM_THRUST0, phBSM2, 2000);
		thBSM[5] = CreateThruster(_V(0.602, 1.708, 23.3), _V(-0.296, -0.696, -0.5), BSM_THRUST0, phBSM2, 2000);
		thBSM[6] = CreateThruster(_V(0.632, 1.738, -23.0), _V(-0.242, -0.696, 0.707), BSM_THRUST0, phBSM2, 2000);
		thBSM[7] = CreateThruster(_V(0.632, 1.738, -23.0), _V(-0.242, -0.696, 0.707), BSM_THRUST0, phBSM2, 2000);
		
	} 
	else 
	{
		//_V(0.632, 1.738, 23.0)
		thBSM[0] = CreateThruster(_V(-0.632, 1.738, 23.0), _V(0.296, -0.813, -0.5), BSM_THRUST0, phBSM1, 2000);
		thBSM[1] = CreateThruster(_V(-0.632, 1.738, 23.0), _V(0.296, -0.813, -0.5), BSM_THRUST0, phBSM1, 2000);
		thBSM[2] = CreateThruster(_V(-0.632, 1.738, -23.0), _V(0.242, -0.696, 0.707), BSM_THRUST0, phBSM1, 2000);
		thBSM[3] = CreateThruster(_V(-0.632, 1.738, -23.0), _V(0.242, -0.696, 0.707), BSM_THRUST0, phBSM1, 2000);
		
		thBSM[4] = CreateThruster(_V(-0.602, 1.708, 23.3), _V(0.296, -0.813, -0.5), BSM_THRUST0, phBSM2, 2000);
		thBSM[5] = CreateThruster(_V(-0.602, 1.708, 23.3), _V(0.296, -0.813, -0.5), BSM_THRUST0, phBSM2, 2000);
		thBSM[6] = CreateThruster(_V(-0.632, 1.738, -23.0), _V(0.242, -0.696, 0.707), BSM_THRUST0, phBSM2, 2000);
		thBSM[7] = CreateThruster(_V(-0.632, 1.738, -23.0), _V(0.242, -0.696, 0.707), BSM_THRUST0, phBSM2, 2000);
		
	}

	for(int i = 0; i<8; i++)
	{
		AddExhaust(thBSM[i], 2.0, 0.3);
		AddExhaustStream(thBSM[i], &bsm_exhaust);
	}
}

void SDLV_RSRB5::IgniteGroup1()
{
	for(int i = 0; i<4; i++)
	{
		SetThrusterLevel(thBSM[i], 1.0);
	} 
}

void SDLV_RSRB5::IgniteGroup2()
{
	for(int i = 4; i<8; i++)
	{
		SetThrusterLevel(thBSM[i], 1.0);
	} 
}

void SDLV_RSRB5::SetRemainingBurnTime(double fBT)
{
	pSRB->SetRemainingBurnTime(fBT);
}

void SDLV_RSRB5::SetRemainingPropellant(double fPropMass)
{
	SetPropellantMass(phSRB, fPropMass);
}

void SDLV_RSRB5::SetupPostSep()
{
	SetEnableFocus(false);
	SetSize(26.9);
	SetEmptyMass(SDLV_RSRB5_EMPTYMASS);
	SetCrossSections(_V(189.28, 197.0,  23.44));
	SetPMI(_V(195.65, 195.65, 1.81));
	CreateAirfoil(LIFT_HORIZONTAL, _V(0.0, 0.0, -3.0), &SRB_LiftFunction, 53, 0, 0.07);
	CreateAirfoil(LIFT_VERTICAL, _V(0.0, 0.0, -3.0), &SRB_LiftFunction, 53, 0, 0.07);
	if(!phSRB)
	{
		phSRB = CreatePropellantResource(SDLV_RSRB5_PROPELLANTMASS);
	}
	pSRB->Create(_V(0.0, 0.0, -23.0), _V(0.0, 0.0, 1.0), phSRB);
	if(!phBSM1)
	{
		phBSM1 = CreatePropellantResource(200.0);
	}
	if(!phBSM2)
	{
		phBSM2 = CreatePropellantResource(200.0);
	}
	CreateBSMThrusters();
	if(fIsLeft)
	{
		AddMesh(MESHNAME_SDLV_RSRB5_LEFT);
	} else
	{
		AddMesh(MESHNAME_SDLV_RSRB5_RIGHT);
	}
	fTimer = 0.0;
	iState = 0;
}


